library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity top_level is

Port (LED					: out std_logic_vector(7 downto 0);
		DIP					: in std_logic_vector(7 downto 0);
		J9						: in std_logic_vector(64 downto 0);
		J10					: in std_logic_vector(64 downto 0);
		J17					: in std_logic_vector(3 downto 0);
		J5						: in std_logic_vector(7 downto 0);
		SDA   	    		: INOUT  STD_LOGIC;
		SCL	       		: INOUT  STD_LOGIC;
		SPKR   	    		: OUT  STD_LOGIC;
		ACBUS0  	    		: IN  STD_LOGIC; -- TXDEN
		ACBUS1  	    		: IN  STD_LOGIC; -- SLEEP#
		ACBUS2  	    		: IN  STD_LOGIC; -- RXLED#
		ACBUS3  	    		: IN  STD_LOGIC; -- TXLED#
		ADBUS0  	    		: IN  STD_LOGIC; -- TXD
		ADBUS1  	    		: OUT STD_LOGIC; -- RXD
		ADBUS2  	    		: IN  STD_LOGIC; -- RTS#
		ADBUS3  	    		: OUT STD_LOGIC; -- CTS#
		ADBUS4  	    		: IN  STD_LOGIC; -- DTR#
		ADBUS5  	    		: OUT STD_LOGIC; -- DSR#
		ADBUS6  	    		: OUT STD_LOGIC; -- DCD#
		ADBUS7  	    		: OUT STD_LOGIC; -- RI#
		BCBUS0  	    		: IN  STD_LOGIC; -- TXDEN
		BCBUS1  	    		: IN  STD_LOGIC; -- SLEEP#
		BCBUS2  	    		: IN  STD_LOGIC; -- RXLED#
		BCBUS3  	    		: IN  STD_LOGIC; -- TXLED#
		BDBUS0  	    		: IN  STD_LOGIC; -- TXD
		BDBUS1  	    		: OUT STD_LOGIC; -- RXD
		BDBUS2  	    		: IN  STD_LOGIC; -- RTS#
		BDBUS3  	    		: OUT STD_LOGIC; -- CTS#
		BDBUS4  	    		: IN  STD_LOGIC; -- DTR#
		BDBUS5  	    		: OUT STD_LOGIC; -- DSR#
		BDBUS6  	    		: OUT STD_LOGIC; -- DCD#
		BDBUS7  	    		: OUT STD_LOGIC; -- RI#
		CLK					: in std_logic);
end top_level;

architecture Behavioral of top_level is
----------------------------------------------
	component uart_tx
	port (
    i_clk       : in  std_logic;
    i_tx_dv     : in  std_logic;
    i_tx_byte   : in  std_logic_vector(7 downto 0);
    o_tx_active : out std_logic;
    o_tx_serial : out std_logic;
    o_tx_done   : out std_logic
    );
	END component;

	component uart_rx
	port (
    i_clk       : in  std_logic;
    i_rx_serial : in  std_logic;
    o_rx_dv     : out std_logic;
    o_rx_byte   : out std_logic_vector(7 downto 0)
    );
	END component;

	COMPONENT i2c_master
	PORT(
		clk : IN std_logic;
		reset_n : IN std_logic;
		ena : IN std_logic;
		addr : IN std_logic_vector(6 downto 0);
		rw : IN std_logic;
		data_wr : IN std_logic_vector(7 downto 0);    
		sda : INOUT std_logic;
		scl : INOUT std_logic;      
		busy : OUT std_logic;
		data_rd : OUT std_logic_vector(7 downto 0);
		ack_error : OUT std_logic
		);
	END COMPONENT;

	COMPONENT music
	PORT(
		clk : IN std_logic;          
		speaker : OUT std_logic
		);
	END COMPONENT;
---------------------------------------------
signal i_o: std_logic_vector(151 downto 0);
signal timer	: std_logic_vector(35 downto 0);
signal rx_dv,s_1,s_2,s_3,s_4,tx_dv,ena_i2c_write,power_up,reset,tx_dv_en,lock_rx,ftdi_mode,eeprom_mode,io_test_mode,
music_mode,tx_serial,five_sec,sound2,ena_i2c,ena_i2c_read,stop_iic,i2c_busy,rw_i2c,rst_i2c,count_start,
start_iic,rx_serial,CE,five_min,ten_min,rx_data_switch,ok,sound1 :STD_LOGIC;
signal rx_data,i2c_data_rd,i2c_data_wr,o_rx_data,led1,led2,led3,led4,led0,dip1,dip3 :STD_LOGIC_VECTOR(7 DOWNTO 0);
signal count_pwr :STD_LOGIC_VECTOR(18 DOWNTO 0);
signal count_d :STD_LOGIC_VECTOR(24 DOWNTO 0);
signal mode		: std_logic_vector(3 downto 0);
---------------------------------------------
begin

	uarttx: uart_tx
	port map(	i_clk			=> clk,
					i_tx_dv		=> tx_dv,
					i_tx_byte	=> i2c_data_rd,
--					o_tx_active => ,
--					o_tx_done 	=> ,
					o_tx_serial	=> tx_serial);
					
	uartrx: uart_rx
	port map(	i_clk			=> clk,
					i_rx_serial => rx_serial,
					o_rx_dv 		=> rx_dv,
					o_rx_byte	=> o_rx_data);


	Inst_i2c_master: i2c_master PORT MAP(
		clk => clk,
		reset_n => rst_i2c,
		ena => ena_i2c,
		addr => "1010110", -- AT24C1024 EEPROM ADDRESS (set "1010111" for the "second half")
		rw => rw_i2c,
		data_wr => i2c_data_wr,
		busy => i2c_busy,
		data_rd => i2c_data_rd,
--		ack_error => ,
		sda => sda,
		scl => scl
	);

	Inst_music: music PORT MAP(
		clk => clk,
		speaker => sound2
	);
---------------------------------------------------------
reset <= '1';

	process(clk,reset)
	begin
	  if (reset='0') then
    count_pwr <= (others => '0');
  elsif (clk'event and clk = '1') then
      if (CE='0') then
			count_pwr <= count_pwr + '1';
		end if;
		end if;
	end process;
	
CE <= '1' when count_pwr= X"7FFFF" else '0';
power_up <= '1' when count_pwr= X"7FFFE" else '0'; --wait until EEPROM is ready
rx_data_switch  <= power_up or five_sec or five_min or ten_min;

process(rx_data_switch,rx_dv,o_rx_data)
begin
  if (rx_data_switch = '1') then
      rx_data <= "00000000";
  elsif (rx_dv = '1') then
		rx_data <= o_rx_data;
  end if;
end process;

tx_dv <= not i2c_busy and tx_dv_en;
	
s_1 <= '1' when o_rx_data= "00110001" and rx_dv='1' else '0';
s_2 <= '1' when o_rx_data= "00110010" and rx_dv='1'  else '0';
s_3 <= '1' when o_rx_data= "00110011" and rx_dv='1'  else '0';
s_4 <= '1' when o_rx_data= "00110100" and rx_dv='1'  else '0';

	process(clk,rx_dv)
	begin
	  if (rx_dv='1') then
    timer <= (others => '0');
  elsif (clk'event and clk = '1') then 
			timer <= timer + '1';
		end if;
	end process;

five_sec <= '1' when timer= X"1DCD6500" and music_mode = '1' else '0';
five_min <= '1' when timer= X"6FC23AC00" and ftdi_mode = '1' else '0';
ten_min <= '1' when timer= X"DF8475800" and io_test_mode = '1' else '0';

process(s_1,five_min,power_up)
begin
  if (five_min = '1') or (power_up = '1') then
      ftdi_mode <= '0';
  elsif (s_1 = '1') then
		ftdi_mode <= '1';
  end if;
end process;

process(s_2,power_up)
begin
  if (power_up = '1') then
      eeprom_mode <= '0';
  elsif (s_2 = '1') then
		eeprom_mode <= '1';
  end if;
end process;

process(s_3,ten_min,power_up)
begin
  if (ten_min = '1') or (power_up = '1') then
      io_test_mode <= '0';
  elsif (s_3 = '1') then
		io_test_mode <= '1';
  end if;
end process;

process(s_4,five_sec,power_up)
begin
  if (five_sec = '1') or (power_up = '1') then
      music_mode <= '0';
  elsif (s_4 = '1') then
		music_mode <= '1';
  end if;
end process;

mode(0) <= ftdi_mode;
mode(1) <= eeprom_mode;
mode(2) <= io_test_mode;
mode(3) <= music_mode;

process(mode,led1,DIP,led2,led3,led4,led0)
begin
	   case mode is
		when "0001" => led<=led1; dip1<=dip; dip3<=X"FF";
      when "0010" => led<=led2; dip1<=X"FF"; dip3<=X"FF";
		when "0100" => led<=led3; dip1<=X"FF"; dip3<=dip;
		when "1000" => led<=led4; dip1<=X"FF"; dip3<=X"FF";
      when others => led<=led0; dip1<=X"FF"; dip3<=X"FF";
   end case;
end process;

lock_rx <= ftdi_mode or eeprom_mode or io_test_mode or music_mode;
rx_serial <= BDBUS0 when lock_rx = '0' else '1';

process(rx_data)
begin
	   case rx_data is
		when "00000000" => i2c_data_wr <= "00000000"; --welcome screen
      when "00110001" => i2c_data_wr <= "00000111"; --1
		when "00110010" => i2c_data_wr <= "00001110"; --2
		when "00110011" => i2c_data_wr <= "00010101"; --3
		when "00110100" => i2c_data_wr <= "00011100"; --4
      when others => i2c_data_wr <= "00000000"; --any other
   end case;
end process;

start_iic <= rx_dv or power_up or five_sec or five_min or ten_min;

process(start_iic,stop_iic)
begin
  if (stop_iic = '1') then
      count_start <= '0';
  elsif (start_iic = '1') then
		count_start <= '1';
  end if;
end process;

	process(clk,count_start)
	begin
	  if (count_start='0') then
    count_d <= (others => '0');
  elsif (clk'event and clk = '1') then 
			count_d <= count_d + '1';
		end if;
	end process;

stop_iic <= '1' when count_d > X"FFFFFF" and i2c_busy='0' else '0';
ena_i2c_write <= '1' when count_d < X"6000" and count_start='1' else '0';
rw_i2c <= '0' when count_d < X"8000" else '1';
ena_i2c_read <= '1' when count_d > X"A000" else '0';
ena_i2c <= ena_i2c_write or ena_i2c_read;
tx_dv_en <= '1' when count_d > X"C000" else '0';
rst_i2c <= reset;

process(timer(27 downto 24))
begin
   case timer(27 downto 24) is 
      when "0000" => led0 <= "00000001";
		when "0001" => led0 <= "00000001";
		when "0010" => led0 <= "00000010";
		when "0011" => led0 <= "00000100";
		when "0100" => led0 <= "00001000";
		when "0101" => led0 <= "00010000";
		when "0110" => led0 <= "00100000";
		when "0111" => led0 <= "01000000";
		when "1000" => led0 <= "10000000";
		when "1001" => led0 <= "10000000";
		when "1010" => led0 <= "01000000";
		when "1011" => led0 <= "00100000";
		when "1100" => led0 <= "00010000";
		when "1101" => led0 <= "00001000";
		when "1110" => led0 <= "00000100";
		when "1111" => led0 <= "00000010";
      when others => led0 <= "00000000";
   end case;
end process;

BDBUS1 <= ADBUS0 when ftdi_mode='1' and count_start='0' else tx_serial;
ADBUS1 <= BDBUS0;
BDBUS3 <= ADBUS2;
ADBUS3 <= BDBUS2;
BDBUS5 <= ADBUS4;
BDBUS6 <= ADBUS4;
BDBUS7 <= ADBUS4 when dip1(0)='1' else '0';
ADBUS5 <= BDBUS4;
ADBUS6 <= BDBUS4;
ADBUS7 <= BDBUS4 when dip1(1)='1' else '0';

LED1(0) <= ACBUS0;
LED1(1) <= not ACBUS1;
LED1(2) <= not ACBUS2;
LED1(3) <= not ACBUS3;
LED1(4) <= BCBUS0;
LED1(5) <= not BCBUS1;
LED1(6) <= not BCBUS2;
LED1(7) <= not BCBUS3;

LED2(0) <= SCL;
LED2(1) <= SCL;
LED2(2) <= SCL;
LED2(3) <= '0';
LED2(4) <= '0';
LED2(5) <= SDA;
LED2(6) <= SDA;
LED2(7) <= SDA;

i_o(64 downto 0) <= J9;
i_o(129 downto 65) <= J10;
i_o(133 downto 130) <= J17;
i_o(141 downto 134) <= J5;
i_o(149 downto 142) <= not dip3;
i_o(150) <= '0';
i_o(151) <= '0';

process (clk)
BEGIN
if (clk'event and clk='1') then
case i_o(151 downto 0) is
when X"00000000000000000000000000000000000001" => ok <='1';
when X"00000000000000000000000000000000000002" => ok <='1';
when X"00000000000000000000000000000000000004" => ok <='1';
when X"00000000000000000000000000000000000008" => ok <='1';
when X"00000000000000000000000000000000000010" => ok <='1';
when X"00000000000000000000000000000000000020" => ok <='1';
when X"00000000000000000000000000000000000040" => ok <='1';
when X"00000000000000000000000000000000000080" => ok <='1';
when X"00000000000000000000000000000000000100" => ok <='1';
when X"00000000000000000000000000000000000200" => ok <='1';
when X"00000000000000000000000000000000000400" => ok <='1';
when X"00000000000000000000000000000000000800" => ok <='1';
when X"00000000000000000000000000000000001000" => ok <='1';
when X"00000000000000000000000000000000002000" => ok <='1';
when X"00000000000000000000000000000000004000" => ok <='1';
when X"00000000000000000000000000000000008000" => ok <='1';
when X"00000000000000000000000000000000010000" => ok <='1';
when X"00000000000000000000000000000000020000" => ok <='1';
when X"00000000000000000000000000000000040000" => ok <='1';
when X"00000000000000000000000000000000080000" => ok <='1';
when X"00000000000000000000000000000000100000" => ok <='1';
when X"00000000000000000000000000000000200000" => ok <='1';
when X"00000000000000000000000000000000400000" => ok <='1';
when X"00000000000000000000000000000000800000" => ok <='1';
when X"00000000000000000000000000000001000000" => ok <='1';
when X"00000000000000000000000000000002000000" => ok <='1';
when X"00000000000000000000000000000004000000" => ok <='1';
when X"00000000000000000000000000000008000000" => ok <='1';
when X"00000000000000000000000000000010000000" => ok <='1';
when X"00000000000000000000000000000020000000" => ok <='1';
when X"00000000000000000000000000000040000000" => ok <='1';
when X"00000000000000000000000000000080000000" => ok <='1';
when X"00000000000000000000000000000100000000" => ok <='1';
when X"00000000000000000000000000000200000000" => ok <='1';
when X"00000000000000000000000000000400000000" => ok <='1';
when X"00000000000000000000000000000800000000" => ok <='1';
when X"00000000000000000000000000001000000000" => ok <='1';
when X"00000000000000000000000000002000000000" => ok <='1';
when X"00000000000000000000000000004000000000" => ok <='1';
when X"00000000000000000000000000008000000000" => ok <='1';
when X"00000000000000000000000000010000000000" => ok <='1';
when X"00000000000000000000000000020000000000" => ok <='1';
when X"00000000000000000000000000040000000000" => ok <='1';
when X"00000000000000000000000000080000000000" => ok <='1';
when X"00000000000000000000000000100000000000" => ok <='1';
when X"00000000000000000000000000200000000000" => ok <='1';
when X"00000000000000000000000000400000000000" => ok <='1';
when X"00000000000000000000000000800000000000" => ok <='1';
when X"00000000000000000000000001000000000000" => ok <='1';
when X"00000000000000000000000002000000000000" => ok <='1';
when X"00000000000000000000000004000000000000" => ok <='1';
when X"00000000000000000000000008000000000000" => ok <='1';
when X"00000000000000000000000010000000000000" => ok <='1';
when X"00000000000000000000000020000000000000" => ok <='1';
when X"00000000000000000000000040000000000000" => ok <='1';
when X"00000000000000000000000080000000000000" => ok <='1';
when X"00000000000000000000000100000000000000" => ok <='1';
when X"00000000000000000000000200000000000000" => ok <='1';
when X"00000000000000000000000400000000000000" => ok <='1';
when X"00000000000000000000000800000000000000" => ok <='1';
when X"00000000000000000000001000000000000000" => ok <='1';
when X"00000000000000000000002000000000000000" => ok <='1';
when X"00000000000000000000004000000000000000" => ok <='1';
when X"00000000000000000000008000000000000000" => ok <='1';
when X"00000000000000000000010000000000000000" => ok <='1';
when X"00000000000000000000020000000000000000" => ok <='1';
when X"00000000000000000000040000000000000000" => ok <='1';
when X"00000000000000000000080000000000000000" => ok <='1';
when X"00000000000000000000100000000000000000" => ok <='1';
when X"00000000000000000000200000000000000000" => ok <='1';
when X"00000000000000000000400000000000000000" => ok <='1';
when X"00000000000000000000800000000000000000" => ok <='1';
when X"00000000000000000001000000000000000000" => ok <='1';
when X"00000000000000000002000000000000000000" => ok <='1';
when X"00000000000000000004000000000000000000" => ok <='1';
when X"00000000000000000008000000000000000000" => ok <='1';
when X"00000000000000000010000000000000000000" => ok <='1';
when X"00000000000000000020000000000000000000" => ok <='1';
when X"00000000000000000040000000000000000000" => ok <='1';
when X"00000000000000000080000000000000000000" => ok <='1';
when X"00000000000000000100000000000000000000" => ok <='1';
when X"00000000000000000200000000000000000000" => ok <='1';
when X"00000000000000000400000000000000000000" => ok <='1';
when X"00000000000000000800000000000000000000" => ok <='1';
when X"00000000000000001000000000000000000000" => ok <='1';
when X"00000000000000002000000000000000000000" => ok <='1';
when X"00000000000000004000000000000000000000" => ok <='1';
when X"00000000000000008000000000000000000000" => ok <='1';
when X"00000000000000010000000000000000000000" => ok <='1';
when X"00000000000000020000000000000000000000" => ok <='1';
when X"00000000000000040000000000000000000000" => ok <='1';
when X"00000000000000080000000000000000000000" => ok <='1';
when X"00000000000000100000000000000000000000" => ok <='1';
when X"00000000000000200000000000000000000000" => ok <='1';
when X"00000000000000400000000000000000000000" => ok <='1';
when X"00000000000000800000000000000000000000" => ok <='1';
when X"00000000000001000000000000000000000000" => ok <='1';
when X"00000000000002000000000000000000000000" => ok <='1';
when X"00000000000004000000000000000000000000" => ok <='1';
when X"00000000000008000000000000000000000000" => ok <='1';
when X"00000000000010000000000000000000000000" => ok <='1';
when X"00000000000020000000000000000000000000" => ok <='1';
when X"00000000000040000000000000000000000000" => ok <='1';
when X"00000000000080000000000000000000000000" => ok <='1';
when X"00000000000100000000000000000000000000" => ok <='1';
when X"00000000000200000000000000000000000000" => ok <='1';
when X"00000000000400000000000000000000000000" => ok <='1';
when X"00000000000800000000000000000000000000" => ok <='1';
when X"00000000001000000000000000000000000000" => ok <='1';
when X"00000000002000000000000000000000000000" => ok <='1';
when X"00000000004000000000000000000000000000" => ok <='1';
when X"00000000008000000000000000000000000000" => ok <='1';
when X"00000000010000000000000000000000000000" => ok <='1';
when X"00000000020000000000000000000000000000" => ok <='1';
when X"00000000040000000000000000000000000000" => ok <='1';
when X"00000000080000000000000000000000000000" => ok <='1';
when X"00000000100000000000000000000000000000" => ok <='1';
when X"00000000200000000000000000000000000000" => ok <='1';
when X"00000000400000000000000000000000000000" => ok <='1';
when X"00000000800000000000000000000000000000" => ok <='1';
when X"00000001000000000000000000000000000000" => ok <='1';
when X"00000002000000000000000000000000000000" => ok <='1';
when X"00000004000000000000000000000000000000" => ok <='1';
when X"00000008000000000000000000000000000000" => ok <='1';
when X"00000010000000000000000000000000000000" => ok <='1';
when X"00000020000000000000000000000000000000" => ok <='1';
when X"00000040000000000000000000000000000000" => ok <='1';
when X"00000080000000000000000000000000000000" => ok <='1';
when X"00000100000000000000000000000000000000" => ok <='1';
when X"00000200000000000000000000000000000000" => ok <='1';
when X"00000400000000000000000000000000000000" => ok <='1';
when X"00000800000000000000000000000000000000" => ok <='1';
when X"00001000000000000000000000000000000000" => ok <='1';
when X"00002000000000000000000000000000000000" => ok <='1';
when X"00004000000000000000000000000000000000" => ok <='1';
when X"00008000000000000000000000000000000000" => ok <='1';
when X"00010000000000000000000000000000000000" => ok <='1';
when X"00020000000000000000000000000000000000" => ok <='1';
when X"00040000000000000000000000000000000000" => ok <='1';
when X"00080000000000000000000000000000000000" => ok <='1';
when X"00100000000000000000000000000000000000" => ok <='1';
when X"00200000000000000000000000000000000000" => ok <='1';
when X"00400000000000000000000000000000000000" => ok <='1';
when X"00800000000000000000000000000000000000" => ok <='1';
when X"01000000000000000000000000000000000000" => ok <='1';
when X"02000000000000000000000000000000000000" => ok <='1';
when X"04000000000000000000000000000000000000" => ok <='1';
when X"08000000000000000000000000000000000000" => ok <='1';
when X"10000000000000000000000000000000000000" => ok <='1';
when X"20000000000000000000000000000000000000" => ok <='1';
when X"40000000000000000000000000000000000000" => ok <='1';
when X"80000000000000000000000000000000000000" => ok <='1';
when others => ok <= '0';
end case;
end if;
end process;

spkr <= sound1 when music_mode='0' else sound2;
sound1 <= timer(14) and ok and io_test_mode;

led3 <= X"FF" when ok='1' else X"00";
led4 <= X"FF" when timer(23)='1' else X"00";

end Behavioral;